#IF 0  ' 

    Build this code with PBCC5

#ENDIF ' 

FUNCTION PBmain as LONG

    LOCAL iflag as DWORD

    cmd$ = command$

    arg1$ = parse$(cmd$," ",1)
    arg2$ = parse$(cmd$," ",2)

    If arg1$ = "/?" Then
      help
      Exit FUNCTION
    End If

    If lcase$(arg2$) = "/i" Then
      iflag = 1
    Else
      iflag = 0
    End If

    If exist(arg1$) = 0 Then
      If arg1$ = "" Then
        cout "No file specified on command line"
        help
      Else
        cout "Cannot find that file >>> "+arg1$
        help
      End If
      Exit FUNCTION
    End If

    hash$ = sha1_hash_file(arg1$,iflag,16)

    If iflag = 0 Then
      cout hash$
    Else
      cout chr$(13,10)+hash$
    End If

End FUNCTION

' 

SUB help

  helpmsg$ = _
    "SHA1 32 bit hash utility"+chr$(13,10)+_
    chr$(13,10)+_
    "USAGE sha1hash filename.ext [/i] [/?]"+chr$(13,10)+_
    "Options /i show progress indicator in console mode (for very large files)"+chr$(13,10)+_
    "        /? show this help message"

    cout helpmsg$

END SUB

' 

    MACRO ALG_CLASS_HASH  = 32768          ''' 4 SHL 13
    MACRO ALG_TYPE_ANY    = 0
    MACRO ALG_SID_SHA1    = 4
    MACRO CALG_SHA1       = ALG_CLASS_HASH or ALG_TYPE_ANY or ALG_SID_SHA1

    MACRO PROV_RSA_FULL   = 1
    MACRO CRYPT_VERIFYCONTEXT = &HF0000000
    MACRO HP_HASHVAL      = 2
    MACRO NULL            = 0

  ' ------------------------------------------
  ' these API DECLAREs are slightly renamed so
  ' they don't clash with future INCLUDE files
  ' ------------------------------------------

    DECLARE FUNCTION Crypt_Acquire_Context LIB "ADVAPI32.DLL" ALIAS "CryptAcquireContextA"( _
                     ByVal phProv as DWORD,ByVal pszContainer as DWORD, _
                     ByVal pszProvider as DWORD,ByVal dwProvType as DWORD, _
                     ByVal dwFlags as DWORD) as DWORD

    DECLARE FUNCTION Crypt_Create_Hash LIB "ADVAPI32.DLL" ALIAS "CryptCreateHash"( _
                     ByVal hProv as DWORD,ByVal Algid as DWORD, _
                     ByVal hKey as DWORD,ByVal dwFlags as DWORD, _
                     ByVal phHash as DWORD) as DWORD

    DECLARE FUNCTION Crypt_Hash_Data LIB "ADVAPI32.DLL" ALIAS "CryptHashData"( _
                     ByVal hHash as DWORD,ByVal pbData as DWORD, _
                     ByVal dwDataLen as DWORD,ByVal dwFlags as DWORD) as DWORD

    DECLARE FUNCTION Crypt_Get_Hash_Param LIB "ADVAPI32.DLL" ALIAS "CryptGetHashParam"( _
                     ByVal hHash as DWORD,ByVal dwParam as DWORD, _
                     ByVal pbData as DWORD,ByVal pdwDataLen as DWORD, _
                     ByVal dwFlags as DWORD) as DWORD

    DECLARE FUNCTION Crypt_Destroy_Hash LIB "ADVAPI32.DLL" ALIAS "CryptDestroyHash"( _
                     ByVal hHash as DWORD) as DWORD

    DECLARE FUNCTION Crypt_Release_Context LIB "ADVAPI32.DLL" ALIAS "CryptReleaseContext"( _
                     ByVal hProv as DWORD,ByVal dwFlags as DWORD) as DWORD

' -------------------------------------------------

FUNCTION sha1_hash_file(fname$,ByVal indicator as DWORD,OPT ByVal blocksize as DWORD) as STRING

  ' -----------------------------------------------------------
  ' INPUT
  ' arg1 = valid file name (must exist)
  ' arg2 = optional buffer size in megabytes. 10 = 10 megabytes
  '
  ' OUTPUT
  ' 40 byte long SHA1 hash of file passed as 1st argument as basic string
  ' -----------------------------------------------------------

    #REGISTER NONE

    LOCAL psrc as DWORD
    LOCAL lsrc as DWORD
    LOCAL phsh as DWORD
    LOCAL hProv as DWORD
    LOCAL hHash as DWORD
    LOCAL dwDataLen as DWORD
    LOCAL phx as DWORD

    LOCAL hFile as DWORD
    LOCAL flen as DWORD

    hFile = fopen(fname$)               ' create a file handle for file
    flen  = fsize(hFile)                ' get the file size

    If blocksize = 0 Then
      blocksize = 1024*1024*16          ' default 16 meg if not set
    Else
      blocksize = 1024*1024*blocksize   ' user defined buffer size in megabytes
    End If

    hashbuff$ = space$(20)              ' 20 byte working buffer
    phsh = StrPtr(hashbuff$)            ' working buffer pointer
    hxout$ = space$(40)                 ' allocate 40 bytes for the output string
    phx = StrPtr(hxout$)                ' get its address
    dwDataLen = 20                      ' set data length to length of hashbuff$

    Crypt_Acquire_Context VarPtr(hProv),NULL,NULL,PROV_RSA_FULL,CRYPT_VERIFYCONTEXT
    Crypt_Create_Hash hProv, CALG_SHA1, 0, 0, VarPtr(hHash)

  ' *********************************

  lpstart:
    block$ = fread(hFile,blocksize)     ' read each block
    psrc = StrPtr(block$)               ' get its address
    lsrc = len(block$)                  ' get its length

    Crypt_Hash_Data hHash,psrc,lsrc,0   ' hash each block
    ! mov eax, lsrc                     ' add length to pointer
    ! add psrc, eax

    If indicator = 1 Then
      StdOut ".";
    End If

    If len(block$) = blocksize Then     ' loop back if block is still original size
    ! jmp lpstart
    End If

  ' *********************************

    fclose hFile                        ' close the file

  ' ----------------------------
  ' clean up after hash API code
  ' ----------------------------
    Crypt_Get_Hash_Param hHash, HP_HASHVAL, phsh, VarPtr(dwDataLen), 0
    Crypt_Destroy_Hash hHash
    Crypt_Release_Context hProv, 0

  ' --------------------------------------
  ' convert bytes in phsh to hex in hxout$
  ' --------------------------------------
    ! xor ebx, ebx                      ' zero the counter
    ! mov esi, phsh                     ' load the address of bytes in phsh
    ! mov edi, phx                      ' load the output buffer address

  oloop:
    ! movzx eax, BYTE PTR [esi+ebx]
    ! movzx ecx, WORD PTR hextbl[eax*2]
    ! mov [edi], cx
    ! add edi, 2
    ! add ebx, 1
    ! cmp ebx, 20
    ! jle oloop

    FUNCTION = hxout$
    Exit FUNCTION

  #align 4
  hextbl:
    ! dw "00","10","20","30","40","50","60","70","80","90","a0","b0","c0","d0","e0","f0"
    ! dw "01","11","21","31","41","51","61","71","81","91","a1","b1","c1","d1","e1","f1"
    ! dw "02","12","22","32","42","52","62","72","82","92","a2","b2","c2","d2","e2","f2"
    ! dw "03","13","23","33","43","53","63","73","83","93","a3","b3","c3","d3","e3","f3"
    ! dw "04","14","24","34","44","54","64","74","84","94","a4","b4","c4","d4","e4","f4"
    ! dw "05","15","25","35","45","55","65","75","85","95","a5","b5","c5","d5","e5","f5"
    ! dw "06","16","26","36","46","56","66","76","86","96","a6","b6","c6","d6","e6","f6"
    ! dw "07","17","27","37","47","57","67","77","87","97","a7","b7","c7","d7","e7","f7"
    ! dw "08","18","28","38","48","58","68","78","88","98","a8","b8","c8","d8","e8","f8"
    ! dw "09","19","29","39","49","59","69","79","89","99","a9","b9","c9","d9","e9","f9"
    ! dw "0a","1a","2a","3a","4a","5a","6a","7a","8a","9a","aa","ba","ca","da","ea","fa"
    ! dw "0b","1b","2b","3b","4b","5b","6b","7b","8b","9b","ab","bb","cb","db","eb","fb"
    ! dw "0c","1c","2c","3c","4c","5c","6c","7c","8c","9c","ac","bc","cc","dc","ec","fc"
    ! dw "0d","1d","2d","3d","4d","5d","6d","7d","8d","9d","ad","bd","cd","dd","ed","fd"
    ! dw "0e","1e","2e","3e","4e","5e","6e","7e","8e","9e","ae","be","ce","de","ee","fe"
    ! dw "0f","1f","2f","3f","4f","5f","6f","7f","8f","9f","af","bf","cf","df","ef","ff"

End FUNCTION

' 

    DECLARE FUNCTION open_fileA LIB "KERNEL32.DLL" ALIAS "CreateFileA" ( _
                     ByVal lpFileName AS DWORD, _
                     ByVal dwDesiredAccess AS DWORD, _
                     ByVal dwShareMode AS DWORD, _
                     ByVal lpSecurityAttributes AS DWORD, _
                     ByVal dwCreationDisposition AS DWORD, _
                     ByVal dwFlagsAndAttributes AS DWORD, _
                     BYVAL hTemplateFile AS DWORD) AS DWORD

 '     MACRO GENERIC_READ          = &H80000000&
 '     MACRO GENERIC_WRITE         = &H40000000&
 '     MACRO OPEN_EXISTING         = 3
 '     MACRO FILE_ATTRIBUTE_NORMAL = &H00000080

        '     '' FILE_FLAG_SEQUENTIAL_SCAN
 ''' %FILE_FLAG_SEQUENTIAL_SCAN     = &H008000000

    ''' %FILE_ATTRIBUTE_NOT_CONTENT_INDEXED = &H00002000



' -----------------------------

FUNCTION fopen(fname$) as DWORD

    FUNCTION = open_fileA(StrPtr(fname$),&H80000000& or &H40000000&,0,0,3,&H008000000 or &H00002000,0)
                          
END FUNCTION

' 

    DECLARE FUNCTION closef LIB "KERNEL32.DLL" ALIAS "CloseHandle" ( _
                     BYVAL hObject AS DWORD) AS LONG

' --------------------------------------------

FUNCTION fclose(ByVal hFile as DWORD) as DWORD

    FUNCTION = closef(hFile)

END FUNCTION

' 

    DECLARE FUNCTION file_read LIB "KERNEL32.DLL" ALIAS "ReadFile" ( _
                     BYVAL hFile AS DWORD,ByVal pbuff as DWORD, BYVAL nNumberOfBytesToRead AS DWORD, _
                     ByVal lpNumberOfBytesRead AS DWORD, ByVal lpOverlapped AS DWORD) AS LONG

' ----------------------------------------------------------------

FUNCTION fread(ByVal hFile as DWORD,ByVal bcnt as DWORD) as STRING

    LOCAL pdat as DWORD         ' string pointer
    LOCAL bred as DWORD         ' bytes read variable

    buffer$ = nul$(bcnt)
    pdat = StrPtr(buffer$)

    file_read(hFile,pdat,bcnt,VarPtr(bred),0)

    If bred <> bcnt Then
      buffer$ = left$(buffer$,bred)
    End If

    FUNCTION = buffer$

END FUNCTION

' 

    DECLARE FUNCTION file_size LIB "KERNEL32.DLL" ALIAS "GetFileSize" ( _
                     BYVAL hFile AS DWORD, lpFileSizeHigh AS LONG) AS LONG

' -------------------------------------------

FUNCTION fsize(ByVal hFile as DWORD) as DWORD

    FUNCTION = file_size(hFile,0)

END FUNCTION

' 

    DECLARE FUNCTION open_file_A LIB "KERNEL32.DLL" ALIAS "CreateFileA" ( _
                     ByVal lpFileName AS DWORD, _
                     ByVal dwDesiredAccess AS DWORD, _
                     ByVal dwShareMode AS DWORD, _
                     ByVal lpSecurityAttributes AS DWORD, _
                     ByVal dwCreationDisposition AS DWORD, _
                     ByVal dwFlagsAndAttributes AS DWORD, _
                     BYVAL hTemplateFile AS DWORD) AS DWORD

    DECLARE FUNCTION get_file_size LIB "KERNEL32.DLL" ALIAS "GetFileSize" ( _
                     BYVAL hFile AS DWORD, lpFileSizeHigh AS LONG) AS LONG

    DECLARE FUNCTION file__read LIB "KERNEL32.DLL" ALIAS "ReadFile" ( _
                     BYVAL hFile AS DWORD,ByVal pbuff as DWORD, BYVAL nNumberOfBytesToRead AS DWORD, _
                     ByVal lpNumberOfBytesRead AS DWORD, ByVal lpOverlapped AS DWORD) AS LONG

    DECLARE FUNCTION closefh LIB "KERNEL32.DLL" ALIAS "CloseHandle" ( _
                     BYVAL hObject AS DWORD) AS LONG

' ------------------------------------

FUNCTION load_file(fname$) as STRING

    LOCAL hFile as DWORD
    LOCAL flen  as DWORD
    LOCAL pdat  as DWORD         ' string pointer
    LOCAL bred  as DWORD         ' bytes read variable

    hFile = open_file_A(StrPtr(fname$),&H80000000& or &H40000000&,0,0,3,&H00000080,0)
    flen  = get_file_size(hFile,0)

    buffer$ = nul$(flen)
    pdat = StrPtr(buffer$)

    file__read(hFile,pdat,flen,VarPtr(bred),0)

    closefh hFile

    FUNCTION = buffer$

End FUNCTION

' 

    DECLARE FUNCTION cc_out CDECL LIB "MSVCRT.DLL" ALIAS "puts" (BYVAL ptxt AS DWORD) AS DWORD

SUB cout(a$)

  ' ***** Supported escapes *****
  ' \0 = ascii zero             0
  ' \t = tab                    9
  ' \n = newline               10
  ' \r = carriage return       13
  ' \q = double quote          34
  ' \\ = backslash             92
  ' *****************************

    #REGISTER NONE

    LOCAL src as DWORD
    LOCAL dst as DWORD
    LOCAL sln as DWORD

    src = StrPtr(a$)

    ! mov esi, src
    ! mov edi, src

  stlp:
    ! mov al, [esi]
    ! add esi, 1

    ! cmp al, "\"
    ! jne nxt

' ====================================

    ! cmp BYTE PTR [esi], "n"
    ! jne lb1
    ! add esi, 1
    ! mov BYTE PTR [edi], 10
    ! add edi, 1
    ! jmp stlp
  lb1:

    ! cmp BYTE PTR [esi], "r"
    ! jne lb2
    ! add esi, 1
    ! mov BYTE PTR [edi], 13
    ! add edi, 1
    ! jmp stlp
  lb2:

    ! cmp BYTE PTR [esi], "t"
    ! jne lb3
    ! add esi, 1
    ! mov BYTE PTR [edi], 9
    ! add edi, 1
    ! jmp stlp
  lb3:

    ! cmp BYTE PTR [esi], "0"
    ! jne lb4
    ! add esi, 1
    ! mov BYTE PTR [edi], 0
    ! add edi, 1
    ! jmp stlp
  lb4:

    ! cmp BYTE PTR [esi], "\"
    ! jne lb5
    ! add esi, 1
    ! mov BYTE PTR [edi], 92
    ! add edi, 1
    ! jmp stlp
  lb5:

    ! cmp BYTE PTR [esi], "q"
    ! jne lb6
    ! add esi, 1
    ! mov BYTE PTR [edi], 34
    ! add edi, 1
    ! jmp stlp
  lb6:

' ====================================

  nxt:
    ! mov [edi], al
    ! add edi, 1
    ! test al, al
    ! jnz stlp

    ! sub edi, src
    ! mov sln, edi

    cesc$ = left$(a$,sln)

    cc_out StrPtr(cesc$)

END SUB

' 

FUNCTION exist(fname$) as DWORD
  FUNCTION = Len( Dir$(fname$, 17) ) > 0
END FUNCTION

' 


 '     LOCAL err1 as LONG
 '     err1 = GetLastError
 '     emsg$ = nul$(64)
 '     FormatMessage %FORMAT_MESSAGE_FROM_SYSTEM,ByVal %NULL,err1,0,ByVal StrPtr(emsg$),64, ByVal %NULL
 '     StdOut emsg$

 '     ! xor ebx, ebx
 '     ! mov edi, phsh
 ' 
 '   lbl0:
 '     ! movzx edx, BYTE PTR [edi+ebx]
 '     ! mov var, edx
 '     ! pushad
 '       hx$ = hx$ + hex$(var,2)
 '     ! popad
 '     ! add ebx, 1
 '     ! cmp ebx, 20
 '     ! jne lbl0

#IF 0  ' 

FUNCTION sha1_hash(src$) as STRING

    #REGISTER NONE

    LOCAL psrc as DWORD
    LOCAL phsh as DWORD
    LOCAL hProv as DWORD
    LOCAL hHash as DWORD
    LOCAL dwDataLen as DWORD
    LOCAL phx as DWORD

    psrc = StrPtr(src$)         ' source address
    hashbuff$ = space$(20)      ' 20 byte working buffer
    phsh = StrPtr(hashbuff$)    ' working buffer pointer
    hxout$ = space$(40)         ' allocate 40 bytes for the output string
    phx = StrPtr(hxout$)        ' get its address
    dwDataLen = 20              ' set data length to length of hashbuff$

    Crypt_Acquire_Context VarPtr(hProv),NULL,NULL,PROV_RSA_FULL,CRYPT_VERIFYCONTEXT
    Crypt_Create_Hash hProv, CALG_SHA1, 0, 0, VarPtr(hHash)
    Crypt_Hash_Data hHash,psrc,len(src$),0
    Crypt_Get_Hash_Param hHash, HP_HASHVAL, phsh, VarPtr(dwDataLen), 0
    Crypt_Destroy_Hash hHash
    Crypt_Release_Context hProv, 0

  ' --------------------------------------
  ' convert bytes in phsh to hex in hxout$
  ' --------------------------------------
    ! xor ebx, ebx              ' zero the counter
    ! mov esi, phsh             ' load the address of bytes in phsh
    ! mov edi, phx              ' load the output buffer address

  oloop:
    ! movzx eax, BYTE PTR [esi+ebx]
    ! movzx ecx, WORD PTR hextbl[eax*2]
    ! mov [edi], cx
    ! add edi, 2
    ! add ebx, 1
    ! cmp ebx, 20
    ! jle oloop

    FUNCTION = hxout$
    Exit FUNCTION

  #align 4
  hextbl:
    ! dw "00","10","20","30","40","50","60","70","80","90","a0","b0","c0","d0","e0","f0"
    ! dw "01","11","21","31","41","51","61","71","81","91","a1","b1","c1","d1","e1","f1"
    ! dw "02","12","22","32","42","52","62","72","82","92","a2","b2","c2","d2","e2","f2"
    ! dw "03","13","23","33","43","53","63","73","83","93","a3","b3","c3","d3","e3","f3"
    ! dw "04","14","24","34","44","54","64","74","84","94","a4","b4","c4","d4","e4","f4"
    ! dw "05","15","25","35","45","55","65","75","85","95","a5","b5","c5","d5","e5","f5"
    ! dw "06","16","26","36","46","56","66","76","86","96","a6","b6","c6","d6","e6","f6"
    ! dw "07","17","27","37","47","57","67","77","87","97","a7","b7","c7","d7","e7","f7"
    ! dw "08","18","28","38","48","58","68","78","88","98","a8","b8","c8","d8","e8","f8"
    ! dw "09","19","29","39","49","59","69","79","89","99","a9","b9","c9","d9","e9","f9"
    ! dw "0a","1a","2a","3a","4a","5a","6a","7a","8a","9a","aa","ba","ca","da","ea","fa"
    ! dw "0b","1b","2b","3b","4b","5b","6b","7b","8b","9b","ab","bb","cb","db","eb","fb"
    ! dw "0c","1c","2c","3c","4c","5c","6c","7c","8c","9c","ac","bc","cc","dc","ec","fc"
    ! dw "0d","1d","2d","3d","4d","5d","6d","7d","8d","9d","ad","bd","cd","dd","ed","fd"
    ! dw "0e","1e","2e","3e","4e","5e","6e","7e","8e","9e","ae","be","ce","de","ee","fe"
    ! dw "0f","1f","2f","3f","4f","5f","6f","7f","8f","9f","af","bf","cf","df","ef","ff"

End FUNCTION

' 

#ENDIF ' 




